/*=======================================================================================================*/
/*      Distributed Speech Recognition                                                                   */
/*      ETSI ES 202 050 Advanced Front-End Feature Extraction Algorithm & Compression Algorithm &        */
/*      ETSI ES 202 212 Extended Advanced Front-End Feature Extraction Algorithm & Compression Algorithm */  
/*      Bit exact C-language software implementation using ETSI basic operations library                 */
/*      3GPP TS 26.243 v 6.1.0 November 2004                                                                  */
/*      ANSI C code for the Fixed-Point Distributed Speech Recognition Extended Advanced Front-end       */
/*=======================================================================================================*/
#ifndef _16kHzPROCESSING_B_H
#define _16kHzPROCESSING_B_H

#define HP16k_MEL_USED       3   /* must be (HP16k_MEL_ORDER - 2) because both first and last halfbands are not used! */
#define NB_LP_BANDS_CODING   3
#define NE16k_FRAMES_THRESH  100

typedef struct DataFor16kProc_B DataFor16kProc_B ;

typedef struct QMF_FIR QMF_FIR;
typedef struct HP_FIL_IIR HP_FIL_IIR;
typedef struct DataFor16kProc_F DataFor16kProc_F ;

/* public functions */

extern void Do16kProcInit_B (FEParamsX_F * This) ;
extern void Do16kProcDelete_B (DataFor16kProc_B *pData16k);
extern void Do16kProcessing_B (X_INT16 *inData_B, DataFor16kProc_B * pData16k, X_INT32 inDataLength) ;
extern struct DataFor16kProc_B *Do16kProcAlloc_B (void) ;


extern void		GetBandsForDecoding16k_B (X_INT32 *intData, X_INT16 in_dec, X_INT32 *bandsForCoding16k, X_INT16 *out_dec, X_INT32 inDataSize);
extern void DecodeBands16k_B (X_INT32 *fb16k_B,X_INT32 *lpBands_B,X_INT32 *codeForBands16k_B, X_INT16 *codeWeights) ;

extern void CodeBands16k_B (X_INT32 *fb16k, X_INT32 *lpBands,X_INT32 *codeForBands16k_B) ;

extern void             DoSpecSub16k_B           (X_INT32 * inFB16k_B, X_INT16 *decal1, DataFor16kProc_B * pData16k, X_INT32 vadCounter16k, X_INT32 inFB16kSize);
extern void             MergeSSandCoded_B        (X_INT32 * intBuffer, X_INT32 * hp16kBands, X_INT32 numChannelsWI7, X_INT16 * hp16kBandsSize_in, struct DataFor16kProc_B * pData16k);
extern X_INT32              CorrectEnergy_B          (X_INT32 LogEnergy, X_INT16 *eng_dec, X_INT32 hp16kBandsSize, X_INT32 * intBuffer, X_INT32 numChannelsWI7, float preem);

extern X_INT16  Get16k_hpBands_dec_B (DataFor16kProc_B *pData16k) ;
extern void Set16k_hpBands_dec_B (DataFor16kProc_B *pData16k, X_INT16 decal) ;
extern X_INT32 Get16k_hpBandsSize_B (DataFor16kProc_B *pData16k);
extern X_INT32 *Get16k_p_CodeForBands16k_B (DataFor16kProc_B *pData16k);
extern X_INT16 *Get16k_p_codeWeights_B (DataFor16kProc_B *pData16k);
extern X_INT32 *Get16k_p_bufferData16k_B (DataFor16kProc_B *pData16k) ;
extern X_INT32 Get16k_bufData16kSize_B (DataFor16kProc_B *pData16k);
extern X_INT32 *Get16k_p_BandsForCoding16k_B (DataFor16kProc_B *pData16k);

extern MelFB_Window *Get16k_p_FirstWindow16k_B (DataFor16kProc_B *pData16k);
extern X_INT32 Get16k_dataHP_B (DataFor16kProc_B *pData16k, int i);

extern X_INT16 * Get16k_BFC_dec_B (DataFor16kProc_B *pData16k);

extern void GetBandsForCoding16k_B (X_INT32 *inData, X_INT16 *nSigSE_dec , X_INT32 *bandsForCoding16k, X_INT16 *decal);

extern X_INT16 Get16k_dec1 (DataFor16kProc_B *pData16k) ;
extern X_INT16 Get16k_dec2 (DataFor16kProc_B *pData16k) ;
extern X_INT16 Get16k_dec3 (DataFor16kProc_B *pData16k) ;
extern X_INT16 Get16k_dec4 (DataFor16kProc_B *pData16k) ;
extern X_INT16 Get16k_dec5 (DataFor16kProc_B *pData16k) ;

extern X_INT16 *Get16k_p_bufferCodeWeights_B (DataFor16kProc_B *pData16k) ;
extern X_INT32 *Get16k_p_bufferCodeForBands16k_B (DataFor16kProc_B *pData16k) ;
extern X_INT32 *Get16k_p_hpBands_B (DataFor16kProc_B *pData16k) ;

#endif
