/*=======================================================================================================*/
/*      Distributed Speech Recognition                                                                   */
/*      ETSI ES 202 050 Advanced Front-End Feature Extraction Algorithm & Compression Algorithm &        */
/*      ETSI ES 202 212 Extended Advanced Front-End Feature Extraction Algorithm & Compression Algorithm */  
/*      Bit exact C-language software implementation using ETSI basic operations library                 */
/*      3GPP TS 26.243 v 6.1.0 November 2004                                                                  */
/*      ANSI C code for the Fixed-Point Distributed Speech Recognition Extended Advanced Front-end       */
/*=======================================================================================================*/

#include <stdio.h>
#include <stdlib.h>

#include "ParmType.h"
#include "typedef.h"
#include "basic_op.h"
#include "count.h"


/*
***********************************************************************
*
*   FUNCTION NAME: qsort_be
*
*   PURPOSE:  sort the given array
*
*   INPUTS:
*
*      iArraySizeQ7 - Array size in Q7
*
*      iElementSizeQ8 - Array element size in Q8
*
*      (*comp) (void *, void *) - Pointer to a comparison function
*
*   INPUT/OUTPUT:
*
*      pvArray[] - Array that is unsorted at input and sorted at output
*
*   OUTPUTS:
*
*      None
*
*   RETURN VALUE:
*
*      None
*
**********************************************************************
*/


/* 
 * Quicksort routine for bit-exact implementation purposes
 *
 * Requires iArraySize to be shifted left by 7 (Q7)
 * Requires iElementSize to be shifted left by 7 (Q7)
 * These requirements impose the following limits:
 *   iElementSize < 256
 *   iArraySize < 512
 */

void qsort_be(void *pvArray, X_INT16 iArraySizeQ7,
              X_INT16 iElementSizeQ7, X_INT16 (*comp)(void *, void *))
{

  X_INT16 i;
  X_INT16 iLast;
  X_INT16 iSize;
  X_INT16 *pcArray;
  Shortword iElementSize;
  void swap(void *, X_INT16, X_INT16, X_INT16);

  test();
  if (sub(iArraySizeQ7,0x0080) <= 0)
  {
    return;
  }

/* size of the array element in Word16 units */
  iElementSize = shr(iElementSizeQ7, 8);

  swap(pvArray,0,(shr(iArraySizeQ7,1)&0xff80),iElementSizeQ7); logic16();

  iLast = 0;					move16();
  iSize = 0;					move16();
  for (i = 0x0080; i < iArraySizeQ7; i+=0x0080)
  {
    iSize = add(iSize,iElementSize);
    pcArray = (X_INT16 *)pvArray + iSize; 
    test();
    if ((*comp) ((void *)pcArray,pvArray) < 0)
    {
      iLast = add(iLast,0x0080);
      swap(pvArray,iLast,i,iElementSizeQ7);
    }
  }

  swap(pvArray,0,iLast,iElementSizeQ7);

  qsort_be(pvArray,iLast,iElementSizeQ7,comp);

  iSize = mult(add(iLast, 0x0080), iElementSizeQ7);

  pcArray = (X_INT16 *) pvArray + iSize;
  qsort_be((void *)pcArray,sub(iArraySizeQ7,add(iLast,0x0080)),iElementSizeQ7,comp);

}


/*
 * Private function
 */

/*
***********************************************************************
*
*   FUNCTION NAME: swap
*
*   PURPOSE: To swap the given pair of array elements
*
*   INPUTS:
*
*      i1 - index of the first array element
*
*      i2 - index of the second array element
*
*      iElementSize - Array element size
*
*   INPUT/OUTPUT:
*
*      pv[] - Array in which the pair is swapped
*
*   OUTPUTS:
*
*      None
*
*   RETURN VALUE:
*
*      None
*
**********************************************************************
*/


/*
 * Optimized swap routine
 * Requires i1, i2 to be shifted left by 7 (Q7)
 * Requires iElementSize to be shifted left by 8 (Q8)
 * These requirements impose the following limits:
 *   iElementSize < 256
 *   i1 < 512
 *   i2 < 512
 */
    
void swap(void *pv, X_INT16 i1, X_INT16 i2, X_INT16 iElementSize)
{

  X_INT16 j;
  X_INT16 j1;
  X_INT16 j2;

  X_INT16 cTemp;
  X_INT16 *pc;

  test();
  if(sub(i1,i2)==0)
  {
      return;
  }

  pc = (X_INT16 *)pv;
  j1 = mult(i1, iElementSize);
  j2 = mult(i2, iElementSize);
  for (j = 0; j < iElementSize; j+=0x0100,j1++,j2++)
  {
    cTemp = *(pc+j1);				move16();
    *(pc+j1) = *(pc+j2);			move16();
    *(pc+j2) = cTemp;				move16();
  }
}


