/*=======================================================================================================*/
/*      Distributed Speech Recognition                                                                   */
/*      ETSI ES 202 050 Advanced Front-End Feature Extraction Algorithm & Compression Algorithm &        */
/*      ETSI ES 202 212 Extended Advanced Front-End Feature Extraction Algorithm & Compression Algorithm */  
/*      Bit exact C-language software implementation using ETSI basic operations library                 */
/*      3GPP TS 26.243 v 6.1.0 November 2004                                                                  */
/*      ANSI C code for the Fixed-Point Distributed Speech Recognition Extended Advanced Front-end       */
/*=======================================================================================================*/
/*-------------------------------------------------------------------------------
 *
 * FILE NAME: 
 *
 *   common.h
 *
 * PURPOSE:	
 *
 *   Shared compiler macros and structures among coder.c and decoder.c
 *
 *-------------------------------------------------------------------------------*/


#ifndef TRUE
#define TRUE (1==1)
#define FALSE !TRUE
#endif

/*************************/
/* multiframe generation */
/*************************/

#define SYNC_SEQ  0x87b2
#define INV_SYNC_SEQ  0x784d
#define ONE_FRAME 0
#define TWO_FRAMES !ONE_FRAME
#define eight_kHz  0x0
#define eleven_kHz 0x1
#define sixteen_kHz 0x3
#define standard  0
#define last_frame 1
#define error -1
#define feType  0x0


/***********************************/
/* Constants for HTK file handling */
/***********************************/

#define EXPECTED_SAMP_PERIOD 100000  /* 100,000  100ns = 10,000 us */
#define EXPECTED_SAMP_SIZE 56  /* 14 * 4 byte floats */
#define EXPECTED_PARM_KIND 8262 /* HTK header parameter type MFCC_E_0 */


/***********************************************/
/* Constants used by reconstruction extensions */
/***********************************************/

#ifdef XAFE

#ifndef MIN_PERIOD
#define                 MIN_PERIOD              1245184
#endif
#ifndef MAX_PERIOD
#define                 MAX_PERIOD              9175040
#endif
#define                 NUM_MULTI_LEVELS_1      26
#define                 NUM_MULTI_LEVELS_2      24
#define                 UNVOICED_CODE           0

#endif



/************************************/
/* Structures for HTK file handling */
/************************************/

typedef struct HTK_DataFrame{
                          float c1;
                          float c2;
                          float c3;
                          float c4;
                          float c5;
                          float c6;
                          float c7;
                          float c8;
                          float c9;
                          float c10;
                          float c11;
                          float c12;
                          float c0;
                          float logE;
			  int VAD;
                          }HTK_DataFrame;

typedef struct HTK_Header{
                          unsigned int nSamples;
                          unsigned int sampPeriod;
                          unsigned short int sampSize;
                          unsigned short int parmKind;
                          }HTK_Header;


/*******************************/
/* required for WMOPS counting */
/*******************************/
#define VQFRAMERATEFACTOR ((1/20e-3)/1e6) /* each frame pair is 20ms */
